"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs = require("fs");
const os_1 = require("os");
const next = require("next");
const axios_1 = require("axios");
const FormData = require('form-data');
const BOTONIC_CLIENT_ID = process.env.BOTONIC_CLIENT_ID || 'jOIYDdvcfwqwSs7ZJ1CpmTKcE7UDapZDOSobFmEp';
const BOTONIC_CLIENT_SECRET = process.env.BOTONIC_CLIENT_SECRET || 'YY34FaaNMnIVKztd6LbLIKn3wFqtiLhDgl6ZVyICwsLVWkZN9UzXw0GXFMmWinP3noNGU9Obtb6Nrr1BwMc4IlCTcRDOKJ96JME5N02IGnIY62ZUezMgfeiUZUmMSu68';
const BOTONIC_URL = process.env.BOTONIC_URL || 'https://api.hubtype.com';
class Botonic {
    constructor(config_path) {
        this.current_path = process.cwd();
        this.bot_path = path_1.join(this.current_path, '/.botonic.json');
        this.df_session_id = Math.random();
        this.path = config_path;
        this.conf = require(path_1.join(this.path, '/.next/botonic.config.js'));
        process.chdir(this.path);
        this.app = next({ dev: false });
    }
    getAction(input) {
        let route = this.conf.routes.find((r) => Object.entries(r)
            .filter(([key, value]) => key != 'action')
            .every(([key, value]) => this.matchRoute(key, value, input)));
        return route ? route.action : '404';
    }
    matchRoute(prop, matcher, input) {
        /*
          prop: ('text' | 'payload' | 'intent')
          matcher: (string: exact match | regex: regular expression match | function: return true)
          input: user input object, ex: {type: 'text', data: 'Hi'}
        */
        let value = '';
        if (prop in input)
            value = input[prop];
        else if (prop == 'text')
            if (input.type == 'text')
                value = input.data;
        if (input.type == 'postback')
            value = input.payload;
        if (typeof matcher === 'string')
            return value == matcher;
        if (matcher instanceof RegExp)
            return matcher.test(value);
        if (typeof matcher === 'function')
            return matcher(value);
        return false;
    }
    async processInput(input) {
        if (input.type == 'text') {
            let intent = await this.getIntent(input);
            if (intent)
                input.intent = intent.data.result.action;
        }
        let component = 'actions/' + this.getAction(input);
        const req = { headers: {}, method: 'GET', url: component };
        const res = {};
        const pathname = component;
        const query = {};
        return this.app.renderToHTML(req, res, pathname, query, {});
    }
    async getIntent(input) {
        if (this.conf.integrations && this.conf.integrations.dialogflow) {
            return axios_1.default({
                headers: {
                    Authorization: 'Bearer ' + this.conf.integrations.dialogflow.token
                },
                url: 'https://api.dialogflow.com/v1/query',
                params: {
                    query: input.data, lang: 'en', sessionId: this.df_session_id
                }
            });
        }
    }
}
exports.Botonic = Botonic;
class BotonicAPIService {
    constructor() {
        this.cliendId = BOTONIC_CLIENT_ID;
        this.clientSecret = BOTONIC_CLIENT_SECRET;
        this.baseUrl = BOTONIC_URL;
        this.baseApiUrl = this.baseUrl + '/v1/';
        this.loginUrl = this.baseUrl + '/o/token/';
        this.botPath = process.cwd();
        this.botCredentialsPath = path_1.join(this.botPath, '/.botonic.json');
        this.globalConfigPath = path_1.join(os_1.homedir(), '/.botonic');
        this.globalCredentialsPath = path_1.join(this.globalConfigPath, '/credentials.json');
        this.headers = null;
        this.loadGlobalCredentials();
        this.loadBotCredentials();
    }
    beforeExit() {
        this.saveGlobalCredentials();
        this.saveBotCredentials();
    }
    loadGlobalCredentials() {
        try {
            var credentials = JSON.parse(fs.readFileSync(this.globalCredentialsPath, 'utf8'));
            var mixpanel = credentials.mixpanel ? credentials.mixpanel : credentials.me.campaign.mixpanel_id;
        }
        catch (e) { }
        if (credentials) {
            this.oauth = credentials.oauth;
            this.me = credentials.me;
            this.mixpanel = mixpanel;
            if (credentials.oauth)
                this.headers = { Authorization: `Bearer ${this.oauth.access_token}` };
        }
    }
    loadBotCredentials() {
        try {
            var credentials = JSON.parse(fs.readFileSync(this.botCredentialsPath, 'utf8'));
        }
        catch (e) { }
        if (credentials)
            this.bot = credentials;
    }
    async checkGlobalCredentialsPath() {
        if (!fs.existsSync(this.globalConfigPath))
            fs.mkdirSync(this.globalConfigPath);
    }
    async saveGlobalCredentials() {
        await this.checkGlobalCredentialsPath();
        fs.writeFileSync(this.globalCredentialsPath, JSON.stringify({
            oauth: this.oauth,
            me: this.me,
            mixpanel: this.mixpanel
        }));
    }
    saveBotCredentials() {
        fs.writeFileSync(this.botCredentialsPath, JSON.stringify(this.bot));
    }
    setCurrentBot(bot) {
        this.bot = bot;
    }
    setMixpanelInfo(mixpanel_id) {
        this.mixpanel = { 'mixpanel_id': mixpanel_id };
    }
    logout() {
        if (fs.existsSync(this.globalCredentialsPath))
            fs.unlinkSync(this.globalCredentialsPath);
    }
    async api(path, body = null, method = 'get', headers = null, params = null) {
        return axios_1.default({
            method: method,
            url: this.baseApiUrl + path,
            headers: headers || this.headers,
            data: body,
            params: params
        });
    }
    async login(email, password) {
        let resp = await axios_1.default({
            method: 'post',
            url: this.loginUrl,
            params: {
                'username': email,
                'password': password,
                'client_id': this.cliendId,
                'client_secret': this.clientSecret,
                'grant_type': 'password'
            }
        });
        this.oauth = resp.data;
        this.headers = { Authorization: `Bearer ${this.oauth.access_token}` };
        resp = await this.api('users/me');
        if (resp)
            this.me = resp.data;
        return resp;
    }
    signup(email, password, org_name, campaign) {
        let url = `${this.baseApiUrl}users/`;
        if (campaign)
            campaign.mixpanel_id = this.mixpanel ? this.mixpanel.distinct_id : Math.round(Math.random() * 10000000000);
        let signup_data = { email, password, org_name, campaign };
        return axios_1.default({
            method: 'post',
            url: url,
            data: signup_data
        });
    }
    async saveBot(bot_name) {
        let resp = await this.api('bots/', { name: bot_name, framework: 'framework_botonic' }, 'post');
        if (resp.data)
            this.setCurrentBot(resp.data);
        return resp;
    }
    async getBots() {
        return this.api('bots/', null, 'get', null, { organization_id: this.me.organization_id });
    }
    async getProviders() {
        return this.api('provider_accounts/', null, 'get', null, { bot_id: this.bot.id });
    }
    async deployBot(bundlePath, password) {
        const form = new FormData();
        let data = fs.createReadStream(bundlePath);
        form.append('bundle', data, 'botonic_bundle.zip');
        let headers = await this.getHeaders(form);
        return this.api(`bots/${this.bot.id}/deploy_botonic/`, form, 'post', Object.assign({}, this.headers, headers), { password: password });
    }
    async getHeaders(form) {
        //https://github.com/axios/axios/issues/1006#issuecomment-352071490
        return new Promise((resolve, reject) => {
            form.getLength((err, length) => {
                if (err) {
                    reject(err);
                }
                let headers = Object.assign({ 'Content-Length': length }, form.getHeaders());
                resolve(headers);
            });
        });
    }
}
exports.BotonicAPIService = BotonicAPIService;
