"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const path_1 = require("path");
const inquirer_1 = require("inquirer");
const fs = require("fs");
const utils_1 = require("../utils");
const util = require('util');
const ora = require('ora');
const exec = util.promisify(require('child_process').exec);
class Run extends command_1.Command {
    constructor() {
        super(...arguments);
        //private templates: any = ['introduction_bot', 'basic_bot', 'basic_actions', 'AI_bot']
        this.templates = [{
                name: 'tutorial',
                description: 'Tutorial: A template with different examples that help you get started fast'
            }, {
                name: 'blank',
                description: 'Blank: A minimal template to start from scratch'
            }];
    }
    async run() {
        utils_1.track('botonic_new');
        const { args, flags } = this.parse(Run);
        let template = '';
        if (!args.templateName) {
            await this.selectBotName().then((resp) => {
                template = this.templates.filter((t) => t.description === resp.botName)[0].name;
            });
        }
        else {
            let botExists = this.templates.filter((t) => t.name === args.templateName);
            if (botExists.length) {
                template = args.templateName;
            }
            else {
                let template_names = this.templates.map((t) => t.name);
                console.log('Template ${args.templateName} does not exist, please choose one of ${template_names}.'.red);
                return;
            }
        }
        if (!fs.existsSync(args.name))
            fs.mkdir(args.name, err => { if (err)
                console.log(err); });
        let botPath = path_1.resolve(template);
        let templatePath = `${__dirname}/../../templates/${template}`;
        let spinner = new ora({
            text: 'Copying files...',
            spinner: 'bouncingBar'
        }).start();
        let copyFolderCommand = `cp -a ${templatePath}/* ${args.name}`;
        let copy_out = await exec(copyFolderCommand);
        spinner.succeed();
        spinner = new ora({
            text: 'Installing dependencies...',
            spinner: 'bouncingBar'
        }).start();
        let dependencyCommand = `cd ${args.name}; npm install`;
        let dependency = await exec(dependencyCommand);
        spinner.succeed();
        spinner = new ora({
            text: 'Compiling...',
            spinner: 'bouncingBar'
        }).start();
        let compileCommand = `cd ${args.name}; npm run build`;
        let compile = await exec(compileCommand);
        spinner.succeed();
        let cd_cmd = `cd ${args.name.bold}`.bold;
        let run_cmd = 'botonic run'.bold;
        let deploy_cmd = 'botonic deploy'.bold;
        console.log(`\n✨  Bot ${args.name.bold} was successfully created!\n\nNext steps:\n${cd_cmd}\n${run_cmd} (test your bot on your terminal)\n${deploy_cmd} (publish your bot to the world!)`);
    }
    async selectBotName() {
        return inquirer_1.prompt([{
                type: 'list',
                name: 'botName',
                message: 'Select a bot template',
                choices: this.templates.map((t) => t.description)
            }]);
    }
}
Run.description = 'Create a new Botonic project';
Run.examples = [
    `$ botonic new test_bot
Creating...
✨ test_bot was successfully created!
`,
];
Run.args = [
    { name: 'name', description: 'name of the bot folder', required: true },
    { name: 'templateName', description: 'OPTIONAL name of the bot template', required: false },
];
exports.default = Run;
